@@ -196,6 +196,64 @@ class ActionDefinition(Definition):
     attributes = sa.Column(st.JsonDictType())
 
 
+class CodeSource(mb.MistralSecureModelBase):
+    """Contains info about registered CodeSources."""
+
+    __tablename__ = 'code_sources'
+    __table_args__ = (
+        sa.UniqueConstraint(
+            'name',
+            'namespace',
+            'project_id'
+        ),
+        sa.Index('%s_project_id' % __tablename__, 'project_id'),
+        sa.Index('%s_scope' % __tablename__, 'scope'),
+    )
+
+    # Main properties.
+    id = mb.id_column()
+    name = sa.Column(sa.String(255))
+    content = sa.Column(sa.Text())
+    version = sa.Column(sa.Integer())
+    namespace = sa.Column(sa.String(255), nullable=True)
+    tags = sa.Column(st.JsonListType())
+
+
+class DynamicActionDefinition(mb.MistralSecureModelBase):
+    """Contains info about registered Dynamic Actions."""
+
+    __tablename__ = 'dynamic_action_definitions'
+    __table_args__ = (
+        sa.UniqueConstraint(
+            'name',
+            'namespace',
+            'project_id'
+        ),
+        sa.Index('%s_project_id' % __tablename__, 'project_id'),
+        sa.Index('%s_scope' % __tablename__, 'scope'),
+    )
+
+    # Main properties.
+    id = mb.id_column()
+    name = sa.Column(sa.String(255))
+    namespace = sa.Column(sa.String(255), nullable=True)
+    class_name = sa.Column(sa.String(255))
+    code_source_name = sa.Column(sa.String(255))
+
+
+DynamicActionDefinition.code_source_id = sa.Column(
+    sa.String(36),
+    sa.ForeignKey(CodeSource.id, ondelete='CASCADE'),
+    nullable=False
+)
+
+DynamicActionDefinition.code_source = relationship(
+    CodeSource,
+    remote_side=CodeSource.id,
+    lazy='select'
+)
+
+
 # Execution objects.
 
 class Execution(mb.MistralSecureModelBase):
