@@ -30,17 +30,11 @@ class WorkflowCancelTest(base.EngineTestCase):
           tasks:
             task1:
               action: std.echo output="Echo"
-              on-complete:
-                - task2
-
-            task2:
-              action: std.echo output="foo"
-              wait-before: 3
         """
 
         wf_service.create_workflows(workflow)
 
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf')
 
         self.engine.stop_workflow(
             wf_ex.id,
@@ -50,27 +44,54 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         self.await_workflow_cancelled(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-        task_1_ex = self._assert_single_item(
-            wf_ex.task_executions,
-            name='task1'
-        )
+            task_execs = wf_ex.task_executions
+
+        task_1_ex = self._assert_single_item(task_execs, name='task1')
 
         self.await_task_success(task_1_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-        task_1_ex = self._assert_single_item(
-            wf_ex.task_executions,
-            name='task1'
-        )
+            task_execs = wf_ex.task_executions
+
+        task_1_ex = self._assert_single_item(task_execs, name='task1')
 
         self.assertEqual(states.CANCELLED, wf_ex.state)
         self.assertEqual("Cancelled by user.", wf_ex.state_info)
-        self.assertEqual(1, len(wf_ex.task_executions))
+        self.assertEqual(1, len(task_execs))
         self.assertEqual(states.SUCCESS, task_1_ex.state)
 
+    def test_cancel_workflow_if_definition_deleted(self):
+        workflow = """
+        version: '2.0'
+
+        wf:
+          type: direct
+          tasks:
+            task1:
+              action: std.echo output="foo"
+              wait-before: 5
+        """
+
+        wf = wf_service.create_workflows(workflow)[0]
+
+        wf_ex = self.engine.start_workflow('wf')
+
+        with db_api.transaction():
+            db_api.delete_workflow_definition(wf.id)
+
+        self.engine.stop_workflow(
+            wf_ex.id,
+            states.CANCELLED,
+            "Cancelled by user."
+        )
+
+        self.await_workflow_cancelled(wf_ex.id)
+
     def test_cancel_paused_workflow(self):
         workflow = """
         version: '2.0'
@@ -80,17 +101,11 @@ class WorkflowCancelTest(base.EngineTestCase):
           tasks:
             task1:
               action: std.echo output="Echo"
-              on-complete:
-                - task2
-
-            task2:
-              action: std.echo output="foo"
-              wait-before: 3
         """
 
         wf_service.create_workflows(workflow)
 
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf')
 
         self.engine.pause_workflow(wf_ex.id)
 
@@ -104,25 +119,28 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         self.await_workflow_cancelled(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-        task_1_ex = self._assert_single_item(
-            wf_ex.task_executions,
-            name='task1'
-        )
+            task_execs = wf_ex.task_executions
+
+        task_1_ex = self._assert_single_item(task_execs, name='task1')
 
         self.await_task_success(task_1_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
 
         task_1_ex = self._assert_single_item(
-            wf_ex.task_executions,
+            task_execs,
             name='task1'
         )
 
         self.assertEqual(states.CANCELLED, wf_ex.state)
         self.assertEqual("Cancelled by user.", wf_ex.state_info)
-        self.assertEqual(1, len(wf_ex.task_executions))
+        self.assertEqual(1, len(task_execs))
         self.assertEqual(states.SUCCESS, task_1_ex.state)
 
     def test_cancel_completed_workflow(self):
@@ -138,7 +156,7 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         wf_service.create_workflows(workflow)
 
-        wf_ex = self.engine.start_workflow('wf', {})
+        wf_ex = self.engine.start_workflow('wf')
 
         self.await_workflow_success(wf_ex.id)
 
@@ -148,16 +166,16 @@ class WorkflowCancelTest(base.EngineTestCase):
             "Cancelled by user."
         )
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-        task_1_ex = self._assert_single_item(
-            wf_ex.task_executions,
-            name='task1'
-        )
+            task_execs = wf_ex.task_executions
+
+        task_1_ex = self._assert_single_item(task_execs, name='task1')
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
-        self.assertEqual(1, len(wf_ex.task_executions))
+        self.assertEqual(1, len(task_execs))
         self.assertEqual(states.SUCCESS, task_1_ex.state)
 
     def test_cancel_parent_workflow(self):
@@ -178,17 +196,11 @@ class WorkflowCancelTest(base.EngineTestCase):
               tasks:
                 task1:
                   action: std.echo output="Echo"
-                  on-complete:
-                    - task2
-
-                task2:
-                  action: std.echo output="foo"
-                  wait-before: 2
         """
 
         wb_service.create_workbook_v2(workbook)
 
-        wf_ex = self.engine.start_workflow('wb.wf', {})
+        wf_ex = self.engine.start_workflow('wb.wf')
 
         self.engine.stop_workflow(
             wf_ex.id,
@@ -198,15 +210,21 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         self.await_workflow_cancelled(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-        task_ex = self._assert_single_item(wf_ex.task_executions, name='taskx')
+            task_execs = wf_ex.task_executions
+
+        task_ex = self._assert_single_item(task_execs, name='taskx')
 
         self.await_task_cancelled(task_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-        task_ex = self._assert_single_item(wf_ex.task_executions, name='taskx')
+            task_execs = wf_ex.task_executions
+
+        task_ex = self._assert_single_item(task_execs, name='taskx')
 
         subwf_execs = db_api.get_workflow_executions(
             task_execution_id=task_ex.id
@@ -248,13 +266,17 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         wb_service.create_workbook_v2(workbook)
 
-        self.engine.start_workflow('wb.wf', {})
+        self.engine.start_workflow('wb.wf')
 
-        wf_execs = db_api.get_workflow_executions()
+        with db_api.transaction():
+            wf_execs = db_api.get_workflow_executions()
 
-        wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
-        task_ex = self._assert_single_item(wf_ex.task_executions, name='taskx')
-        subwf_ex = self._assert_single_item(wf_execs, name='wb.subwf')
+            wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
+            task_ex = self._assert_single_item(
+                wf_ex.task_executions,
+                name='taskx'
+            )
+            subwf_ex = self._assert_single_item(wf_execs, name='wb.subwf')
 
         self.engine.stop_workflow(
             subwf_ex.id,
@@ -266,11 +288,15 @@ class WorkflowCancelTest(base.EngineTestCase):
         self.await_task_cancelled(task_ex.id)
         self.await_workflow_cancelled(wf_ex.id)
 
-        wf_execs = db_api.get_workflow_executions()
+        with db_api.transaction():
+            wf_execs = db_api.get_workflow_executions()
 
-        wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
-        task_ex = self._assert_single_item(wf_ex.task_executions, name='taskx')
-        subwf_ex = self._assert_single_item(wf_execs, name='wb.subwf')
+            wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
+            task_ex = self._assert_single_item(
+                wf_ex.task_executions,
+                name='taskx'
+            )
+            subwf_ex = self._assert_single_item(wf_execs, name='wb.subwf')
 
         self.assertEqual(states.CANCELLED, subwf_ex.state)
         self.assertEqual("Cancelled by user.", subwf_ex.state_info)
@@ -307,7 +333,7 @@ class WorkflowCancelTest(base.EngineTestCase):
         """
         wb_service.create_workbook_v2(workbook)
 
-        wf_ex = self.engine.start_workflow('wb.wf', {})
+        wf_ex = self.engine.start_workflow('wb.wf')
 
         self.engine.stop_workflow(
             wf_ex.id,
@@ -315,18 +341,29 @@ class WorkflowCancelTest(base.EngineTestCase):
             "Cancelled by user."
         )
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-        task_ex = self._assert_single_item(wf_ex.task_executions, name='taskx')
+            task_execs = wf_ex.task_executions
+
+        task_ex = self._assert_single_item(task_execs, name='taskx')
 
         self.await_workflow_cancelled(wf_ex.id)
         self.await_task_cancelled(task_ex.id)
 
-        wf_execs = db_api.get_workflow_executions()
+        with db_api.transaction():
+            wf_execs = db_api.get_workflow_executions()
 
-        wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
-        task_ex = self._assert_single_item(wf_ex.task_executions, name='taskx')
-        subwf_exs = self._assert_multiple_items(wf_execs, 2, name='wb.subwf')
+            wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
+            task_ex = self._assert_single_item(
+                wf_ex.task_executions,
+                name='taskx'
+            )
+            subwf_exs = self._assert_multiple_items(
+                wf_execs,
+                2,
+                name='wb.subwf'
+            )
 
         self.assertEqual(states.CANCELLED, subwf_exs[0].state)
         self.assertEqual("Cancelled by user.", subwf_exs[0].state_info)
@@ -366,13 +403,21 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         wb_service.create_workbook_v2(workbook)
 
-        self.engine.start_workflow('wb.wf', {})
+        self.engine.start_workflow('wb.wf')
 
-        wf_execs = db_api.get_workflow_executions()
+        with db_api.transaction():
+            wf_execs = db_api.get_workflow_executions()
 
-        wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
-        task_ex = self._assert_single_item(wf_ex.task_executions, name='taskx')
-        subwf_exs = self._assert_multiple_items(wf_execs, 2, name='wb.subwf')
+            wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
+            task_ex = self._assert_single_item(
+                wf_ex.task_executions,
+                name='taskx'
+            )
+            subwf_exs = self._assert_multiple_items(
+                wf_execs,
+                2,
+                name='wb.subwf'
+            )
 
         self.engine.stop_workflow(
             subwf_exs[0].id,
@@ -385,11 +430,19 @@ class WorkflowCancelTest(base.EngineTestCase):
         self.await_task_cancelled(task_ex.id)
         self.await_workflow_cancelled(wf_ex.id)
 
-        wf_execs = db_api.get_workflow_executions()
+        with db_api.transaction():
+            wf_execs = db_api.get_workflow_executions()
 
-        wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
-        task_ex = self._assert_single_item(wf_ex.task_executions, name='taskx')
-        subwf_exs = self._assert_multiple_items(wf_execs, 2, name='wb.subwf')
+            wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
+            task_ex = self._assert_single_item(
+                wf_ex.task_executions,
+                name='taskx'
+            )
+            subwf_exs = self._assert_multiple_items(
+                wf_execs,
+                2,
+                name='wb.subwf'
+            )
 
         self.assertEqual(states.CANCELLED, subwf_exs[0].state)
         self.assertEqual("Cancelled by user.", subwf_exs[0].state_info)
@@ -429,13 +482,21 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         wb_service.create_workbook_v2(workbook)
 
-        self.engine.start_workflow('wb.wf', {})
+        self.engine.start_workflow('wb.wf')
 
-        wf_execs = db_api.get_workflow_executions()
+        with db_api.transaction():
+            wf_execs = db_api.get_workflow_executions()
 
-        wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
-        task_ex = self._assert_single_item(wf_ex.task_executions, name='taskx')
-        subwf_exs = self._assert_multiple_items(wf_execs, 2, name='wb.subwf')
+            wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
+            task_ex = self._assert_single_item(
+                wf_ex.task_executions,
+                name='taskx'
+            )
+            subwf_exs = self._assert_multiple_items(
+                wf_execs,
+                2,
+                name='wb.subwf'
+            )
 
         self.engine.stop_workflow(
             subwf_exs[0].id,
@@ -451,23 +512,31 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         self.await_workflow_cancelled(subwf_exs[0].id)
         self.await_workflow_error(subwf_exs[1].id)
-        self.await_task_error(task_ex.id)
-        self.await_workflow_error(wf_ex.id)
+        self.await_task_cancelled(task_ex.id)
+        self.await_workflow_cancelled(wf_ex.id)
 
-        wf_execs = db_api.get_workflow_executions()
+        with db_api.transaction():
+            wf_execs = db_api.get_workflow_executions()
 
-        wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
-        task_ex = self._assert_single_item(wf_ex.task_executions, name='taskx')
-        subwf_exs = self._assert_multiple_items(wf_execs, 2, name='wb.subwf')
+            wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
+            task_ex = self._assert_single_item(
+                wf_ex.task_executions,
+                name='taskx'
+            )
+            subwf_exs = self._assert_multiple_items(
+                wf_execs,
+                2,
+                name='wb.subwf'
+            )
 
         self.assertEqual(states.CANCELLED, subwf_exs[0].state)
         self.assertEqual("Cancelled by user.", subwf_exs[0].state_info)
         self.assertEqual(states.ERROR, subwf_exs[1].state)
         self.assertEqual("Failed by user.", subwf_exs[1].state_info)
-        self.assertEqual(states.ERROR, task_ex.state)
-        self.assertIn("failed", task_ex.state_info)
-        self.assertEqual(states.ERROR, wf_ex.state)
-        self.assertIn("Failed by user.", wf_ex.state_info)
+        self.assertEqual(states.CANCELLED, task_ex.state)
+        self.assertIn("cancelled", task_ex.state_info)
+        self.assertEqual(states.CANCELLED, wf_ex.state)
+        self.assertEqual("Cancelled tasks: taskx", wf_ex.state_info)
 
     def test_fail_then_cancel_with_items_child_workflow(self):
         workbook = """
@@ -498,13 +567,21 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         wb_service.create_workbook_v2(workbook)
 
-        self.engine.start_workflow('wb.wf', {})
+        self.engine.start_workflow('wb.wf')
 
-        wf_execs = db_api.get_workflow_executions()
+        with db_api.transaction():
+            wf_execs = db_api.get_workflow_executions()
 
-        wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
-        task_ex = self._assert_single_item(wf_ex.task_executions, name='taskx')
-        subwf_exs = self._assert_multiple_items(wf_execs, 2, name='wb.subwf')
+            wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
+            task_ex = self._assert_single_item(
+                wf_ex.task_executions,
+                name='taskx'
+            )
+            subwf_exs = self._assert_multiple_items(
+                wf_execs,
+                2,
+                name='wb.subwf'
+            )
 
         self.engine.stop_workflow(
             subwf_exs[1].id,
@@ -520,20 +597,28 @@ class WorkflowCancelTest(base.EngineTestCase):
 
         self.await_workflow_cancelled(subwf_exs[0].id)
         self.await_workflow_error(subwf_exs[1].id)
-        self.await_task_error(task_ex.id)
-        self.await_workflow_error(wf_ex.id)
+        self.await_task_cancelled(task_ex.id)
+        self.await_workflow_cancelled(wf_ex.id)
 
-        wf_execs = db_api.get_workflow_executions()
+        with db_api.transaction():
+            wf_execs = db_api.get_workflow_executions()
 
-        wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
-        task_ex = self._assert_single_item(wf_ex.task_executions, name='taskx')
-        subwf_exs = self._assert_multiple_items(wf_execs, 2, name='wb.subwf')
+            wf_ex = self._assert_single_item(wf_execs, name='wb.wf')
+            task_ex = self._assert_single_item(
+                wf_ex.task_executions,
+                name='taskx'
+            )
+            subwf_exs = self._assert_multiple_items(
+                wf_execs,
+                2,
+                name='wb.subwf'
+            )
 
         self.assertEqual(states.CANCELLED, subwf_exs[0].state)
         self.assertEqual("Cancelled by user.", subwf_exs[0].state_info)
         self.assertEqual(states.ERROR, subwf_exs[1].state)
         self.assertEqual("Failed by user.", subwf_exs[1].state_info)
-        self.assertEqual(states.ERROR, task_ex.state)
-        self.assertIn("failed", task_ex.state_info)
-        self.assertEqual(states.ERROR, wf_ex.state)
-        self.assertIn("Failed by user.", wf_ex.state_info)
+        self.assertEqual(states.CANCELLED, task_ex.state)
+        self.assertIn("cancelled", task_ex.state_info)
+        self.assertEqual(states.CANCELLED, wf_ex.state)
+        self.assertEqual("Cancelled tasks: taskx", wf_ex.state_info)
